@@ -305,6 +305,21 @@ class DeviceNDArrayBase(object):
         desc = dict(shape=self.shape, strides=self.strides, dtype=self.dtype)
         return IpcArrayHandle(ipc_handle=ipch, array_desc=desc)
 
+    def view(self, dtype):
+        """Returns a new object by reinterpretting the dtype without making a
+        copy of the data.
+        """
+        dtype = np.dtype(dtype)
+        if dtype.itemsize != self.dtype.itemsize:
+            raise TypeError("new dtype itemsize doesn't match")
+        return DeviceNDArray(
+            shape=self.shape,
+            strides=self.strides,
+            dtype=dtype,
+            stream=self.stream,
+            gpu_data=self.gpu_data,
+            )
+
 
 class DeviceRecord(DeviceNDArrayBase):
     '''
@@ -537,7 +552,6 @@ class DeviceNDArray(DeviceNDArrayBase):
         _assign_kernel(lhs.ndim).forall(n_elements, stream=stream)(lhs, rhs)
 
 
-
 class IpcArrayHandle(object):
     """
     An IPC array handle that can be serialized and transfer to another process
