@@ -63,7 +63,12 @@ class Loc(object):
                     spaces += 1
                 return spaces
 
-            selected = lines[self.line - nlines_up:self.line]
+            # A few places in the code still use no `loc` or default to line 1
+            # this is often in places where exceptions are used for the purposes
+            # of flow control. As a result max is in use to prevent slice from
+            # `[negative: positive]`
+            selected = lines[max(0, self.line - nlines_up):self.line]
+
             # see if selected contains a definition
             def_found = False
             for x in selected:
@@ -83,12 +88,13 @@ class Loc(object):
                     spaces = count_spaces(x)
                     ret.append(' '*(4 + spaces) + '<source elided>\n')
 
-            ret.extend(selected[:-1])
-            ret.append(_termcolor.highlight(selected[-1]))
+            if selected:
+                ret.extend(selected[:-1])
+                ret.append(_termcolor.highlight(selected[-1]))
 
-            # point at the problem with a caret
-            spaces = count_spaces(selected[-1])
-            ret.append(' '*(spaces) + _termcolor.indicate("^"))
+                # point at the problem with a caret
+                spaces = count_spaces(selected[-1])
+                ret.append(' '*(spaces) + _termcolor.indicate("^"))
 
         # if in the REPL source may not be available
         if not ret:
@@ -564,6 +570,31 @@ class Yield(Inst):
         return [self.value]
 
 
+class EnterWith(Stmt):
+    """Enter a "with" context
+    """
+    def __init__(self, contextmanager, begin, end, loc):
+        """
+        Parameters
+        ----------
+        contextmanager : IR value
+        begin, end : int
+            The beginning and the ending offset of the with-body.
+        loc : int
+            Source location
+        """
+        self.contextmanager = contextmanager
+        self.begin = begin
+        self.end = end
+        self.loc = loc
+
+    def __str__(self):
+        return 'enter_with {}'.format(self.contextmanager)
+
+    def list_vars(self):
+        return [self.contextmanager]
+
+
 class Arg(object):
     def __init__(self, name, index, loc):
         self.name = name
@@ -874,6 +905,8 @@ class Block(object):
 
 
 class Loop(object):
+    """Describes a loop-block
+    """
     __slots__ = "entry", "exit"
 
     def __init__(self, entry, exit):
@@ -885,6 +918,20 @@ class Loop(object):
         return "Loop(entry=%s, exit=%s)" % args
 
 
+class With(object):
+    """Describes a with-block
+    """
+    __slots__ = "entry", "exit"
+
+    def __init__(self, entry, exit):
+        self.entry = entry
+        self.exit = exit
+
+    def __repr__(self):
+        args = self.entry, self.exit
+        return "With(entry=%s, exit=%s)" % args
+
+
 class FunctionIR(object):
 
     def __init__(self, blocks, is_generator, func_id, loc,
@@ -1006,4 +1053,8 @@ class FunctionIR(object):
 
 
 # A stub for undefined global reference
-UNDEFINED = object()
+class UndefinedType(object):
+    def __repr__(self):
+        return "Undefined"
+
+UNDEFINED = UndefinedType()
