@@ -12,6 +12,8 @@ from numba import cgutils
 from numba.utils import IS_PY3
 from . import llvm_types as lt
 from numba.compiler import compile_extra, Flags
+from numba.compiler_lock import global_compiler_lock
+
 from numba.targets.registry import cpu_target
 from numba.runtime import nrtdynmod
 
@@ -124,6 +126,7 @@ class _ModuleCompiler(object):
         """
         raise NotImplementedError
 
+    @global_compiler_lock
     def _cull_exports(self):
         """Read all the exported functions/modules in the translator
         environment, and join them into a single LLVM module.
@@ -148,10 +151,10 @@ class _ModuleCompiler(object):
 
         for entry in self.export_entries:
             cres = compile_extra(self.typing_context, self.context,
-                                 entry.function,
-                                 entry.signature.args,
-                                 entry.signature.return_type, flags,
-                                 locals={}, library=library)
+                                entry.function,
+                                entry.signature.args,
+                                entry.signature.return_type, flags,
+                                locals={}, library=library)
 
             func_name = cres.fndesc.llvm_func_name
             llvm_func = cres.library.get_function(func_name)
