@@ -41,6 +41,26 @@ class Integer(Number):
             return NotImplemented
         return self.bitwidth < other.bitwidth
 
+    @property
+    def maxval(self):
+        """
+        The maximum value representable by this type.
+        """
+        if self.signed:
+            return (1 << (self.bitwidth - 1)) - 1
+        else:
+            return (1 << self.bitwidth) - 1
+
+    @property
+    def minval(self):
+        """
+        The minimal value representable by this type.
+        """
+        if self.signed:
+            return -(1 << (self.bitwidth - 1))
+        else:
+            return 0
+
 
 @utils.total_ordering
 class Float(Number):
@@ -85,7 +105,7 @@ class _NPDatetimeBase(Type):
     """
 
     def __init__(self, unit, *args, **kws):
-        name = '%s(%s)' % (self.type_name, unit)
+        name = '%s[%s]' % (self.type_name, unit)
         self.unit = unit
         self.unit_code = npdatetime.DATETIME_UNITS[self.unit]
         super(_NPDatetimeBase, self).__init__(name, *args, **kws)
