@@ -23,6 +23,11 @@ except ImportError:
     from io import StringIO
 from numba.config import PYVERSION, MACHINE_BITS
 
+if PYVERSION >= (3, 3):
+    from collections.abc import Mapping
+else:
+    from collections import Mapping
+
 
 IS_PY3 = PYVERSION >= (3, 0)
 
@@ -33,6 +38,7 @@ if IS_PY3:
     get_ident = threading.get_ident
     intern = sys.intern
     file_replace = os.replace
+    asbyteint = int
 
     def erase_traceback(exc_value):
         """
@@ -49,6 +55,10 @@ else:
     longint = long
     get_ident = thread.get_ident
     intern = intern
+    def asbyteint(x):
+        # convert 1-char str into int
+        return ord(x)
+
     if sys.platform == 'win32':
         def file_replace(src, dest):
             # Best-effort emulation of os.replace()
@@ -228,7 +238,7 @@ class ConfigOptions(object):
         return hash(tuple(sorted(self._values.items())))
 
 
-class SortedMap(collections.Mapping):
+class SortedMap(Mapping):
     """Immutable
     """
 
